const CLASSTYPE: object = {
  '[object Array]': 'array',
  '[object Boolean]': 'boolean',
  '[object Date]': 'date',
  '[object Error]': 'error',
  '[object Function]': 'function',
  '[object Number]': 'number',
  '[object Object]': 'object',
  '[object RegExp]': 'regexp',
  '[object String]': 'string'
}, CLASSHASOWN = CLASSTYPE.hasOwnProperty;

class Cores {
  public isArray: (o: any) => boolean;

  constructor() {
    const _this = this;
    this.isArray = Array.isArray || function(o) {
      return _this.type(o) === 'array';
    };
  }

  // 判断对象是否是空对象
  isEmptyObject(o: object) {
    let a;
    for (a in o) {
      return false;
    }
    return true;
  }

  // 对象是否是方法类型
  isFunction(o: any) {
    return this.type(o) === 'function';
  }

  // 获取函数名
  getFnName(a: any) {
    if (this.isFunction(a)) {
      let s = a.toString().match(/^function\s+(\w+)[^(]+/);
      return s ? s[1] : null;
    }
    return null;
  }

  // 判断对象是否是数值
  isNumeric(o: any) {
    return !isNaN(parseFloat(o)) && isFinite(o);
  }

  // 判断是否普通对象
  isPlainObject(obj: any) {
    let key;
    // 必须是一个对象。
    // 由于IE，还必须检查构造函数属性的存在。
    // 确保DOM节点和window对象也不会通过
    if (!obj || this.type(obj) !== 'object' || obj.nodeType ||
      this.isWindow(obj)) {
      return false;
    }

    try {
      // 排除构造函数
      if (obj.constructor && !CLASSHASOWN.call(obj, 'constructor') &&
        !CLASSHASOWN.call(obj.constructor.prototype, 'isPrototypeOf')) {
        return false;
      }
    } catch (e) {
      // IE8,9 抛出异常
      return false;
    }
    // 首先列举自己的属性，以便加快速度，
    // 如果最后一个属于自己，那么所有属性都是自己的。
    for (key in obj) {
    }
    return key === undefined || CLASSHASOWN.call(obj, key);
  }

  // 判断 promise
  isPromise(e: any) {
    return !!e && typeof e.then === 'function';
  }

  // 判断两个对象是否相等
  isObjectEqual(a: object, b: object) {
    let attr: any;
    for (attr in a) {
      if (a.hasOwnProperty(attr)) {
        if (b.hasOwnProperty(attr)) {
          // @ts-ignore
          if (a[attr] !== b[attr]) {
            return false;
          }
        } else {
          return false;
        }
      }
    }
    for (attr in b) {
      if (b.hasOwnProperty(attr)) {
        if (a.hasOwnProperty(attr)) {
          // @ts-ignore
          if (a[attr] !== b[attr]) {
            return false;
          }
        } else {
          return false;
        }
      }
    }
    return true;
  }

  // 对象拷贝
  extend(...args: any[]) {
    let src, copyIsArray, copy, name, options, clone,
      target = arguments[0] || {},
      i = 1,
      length = arguments.length,
      deep = false;

    // 处理深层复制情况, 第一个参数为是否需要深拷贝
    if (typeof target === 'boolean') {
      deep = target;
      target = arguments[1] || {};
      // 跳过布尔值和目标
      i = 2;
    }

    // 处理目标是字符串或其他内容时的情况（可能是深层复制）
    if (typeof target !== 'object' && !this.isFunction(target)) {
      target = {};
    }

    // 如果只传递一个参数，则扩展本身
    if (length === i) {
      target = this;
      --i;
    }
    for (; i < length; i++) {
      // 仅处理非null 未定义的值
      if ((options = arguments[i]) !== null) {
        // 扩展基础对象
        for (name in options) {
          if (options.hasOwnProperty(name)) {
            src = target[name];
            copy = options[name];

            // 防止死循环
            if (target === copy) {
              continue;
            }

            // 如果我们合并普通对象或数组，则递归
            if (deep && copy && (this.isPlainObject(copy) ||
              (copyIsArray = this.isArray(copy)))) {
              if (copyIsArray) {
                copyIsArray = false;
                clone = src && this.isArray(src) ? src : [];

              } else {
                clone = src && this.isPlainObject(src) ? src : {};
              }

              // 不要移动原始对象，克隆它们
              // @ts-ignore
              target[name] = this.extend(deep, clone, copy);

              // 不要引入未定义的值
            } else if (copy !== undefined) {
              target[name] = copy;
            }
          }
        }
      }
    }

    // 返回修改后的对象
    return target;
  }

  // 遍历方法
  each(obj: any, callback: any, args?: any) {
    if (!obj) {
      return;
    }
    let value,
      i = 0,
      length = obj.length,
      isArray = this.isArraylike(obj);

    if (args) {
      if (isArray) {
        for (; i < length; i++) {
          value = callback.apply(obj[i], args);

          if (value === false) {
            break;
          }
        }
      } else {
        // @ts-ignore
        for (i in obj) {
          if (obj.hasOwnProperty(i)) {
            value = callback.apply(obj[i], args);
            if (value === false) {
              break;
            }
          }
        }
      }

    } else {
      if (isArray) {
        for (; i < length; i++) {
          value = callback.call(obj[i], obj[i], i);

          if (value === false) {
            break;
          }
        }
      } else {
        // @ts-ignore
        for (i in obj) {
          if (obj.hasOwnProperty(i)) {
            value = callback.call(obj[i], obj[i], i);

            if (value === false) {
              break;
            }
          }
        }
      }
    }
    return obj;
  }

  getUrl(name: any) {
    let re = new RegExp('(^|&)' + name + '=([^&]*)(&|$)', 'i');
    let parameter = window.location.search.substr(1).match(re);
    if (parameter !== null) {
      return (parameter[2]);
    } else {
      return null;
    }
  }

  getHash(name: any) {
    let re = new RegExp('(^|[&?])' + name + '=([^&]*)(&|$)', 'i');
    let parameter = window.location.hash.substr(1).match(re);
    if (parameter !== null) {
      return (parameter[2]);
    } else {
      return null;
    }
  }

  routeParams(s: any, y: any) {
    let hash = window.location.hash;
    let reg = /\?([\w\W]+)$/;
    let str: any = hash.match(reg);
    let hashParam: object = {};
    let isNull = true;
    if (str) {
      str = str[1];
      let arr = str.split('&');
      this.each(arr, (d: any) => {
        let a: Array<any> = d.split('=');
        if (a.length > 1) {
          isNull = false;
          if (y === 'array') {
            // @ts-ignore
            hashParam[a[0]] = a[1].split('~');
          } else {
            // @ts-ignore
            hashParam[a[0]] = a[1].indexOf('~') !== -1 ? a[1].split('~') : a[1];
          }
        }
      });
    }
    return s == null && isNull ? null : hashParam;
  }

  // 获取对象类型
  type(o: any) {
    if (o === null || o === undefined) {
      return String(o);
    }
    // @ts-ignore
    return typeof o === 'object' || typeof o === 'function' ? CLASSTYPE[CLASSTYPE.toString.call(o)] || 'object' :
      typeof o;
  }

  // 判断对象是否是window
  isWindow(o: any) {
    return o !== null && o === o.window;
  }

  // 判断对象是否为类数组类型
  isArraylike(obj: any) {
    let length = obj.length,
      objType = this.type(obj);

    if (this.isWindow(obj)) {
      return false;
    }

    if (obj.nodeType === 1 && length) {
      return true;
    }

    return objType === 'array' || objType !== 'function' &&
      (length === 0 || typeof length === 'number' && length > 0 && (length - 1) in obj);
  }

  noop() {
  }

  param(o: any) {
    let keys: Array<any> = [];
    this.buildParams(keys, o);
    return keys.join('&');
  }

  buildParams(keys: any, json: any, attrs?: any) {
    for (let attr in json) {
      if (json.hasOwnProperty(attr)) {
        if (this.type(json[attr]) === 'object' || this.type(json[attr]) === 'array') {
          if (attrs) {
            this.buildParams(keys, json[attr], attrs + '[' + attr + ']');
          } else {
            this.buildParams(keys, json[attr], attr);
          }
        } else {
          let s = json[attr];
          if (this.type(s) === 'undefined' || this.type(s) === 'null') {
            s = '';
          }
          if (attrs) {
            keys.push(encodeURIComponent(attrs + '[' + attr + ']') + '=' + encodeURIComponent(s));
          } else {
            keys.push(encodeURIComponent(attr) + '=' + encodeURIComponent(s));
          }
        }
      }
    }
  }
}

export default new Cores();
