/**
 * 使用方法
 * moment(1478180296130).Format("yyyy-MM-dd E HH:mm:ss") ==> 2016-11-03 四 21:38:04
 * moment(1478180296130).Format("yyyy-MM-dd HH:mm:ss EE") ==> 2016-11-03 21:38:04 周四
 * moment(1478180296130).Format("yyyy-MM-dd HH:mm:ss EEE") ==> 2016-11-0 21:38:04 星期四
 * moment(1478180296130).Format("yyyy-MM-dd HH:mm:ss e") ==> 2016-11-03 21:38:04 4
 * moment(1478180296130).Format("yyyy-MM-dd HH:mm:ss Q") ==> 2016-11-03 21:38:04 冬
 * moment('2016-11-03 21:56:57').Format("X") ==> 1478180296130
 * yyyy ==> 2016   yy ==> 16  代表年
 * MM ==> 12 | 02  M ==> 10 | 2  代表两位数的月份，3月会显示为03
 * dd ==> 24 | 05  d ==> 3  //返回日，单双决定不足双数是否补零
 * HH ==> 23   hh ==> 11  //大写返回24小时制小写返回12小时制
 * mm == 36 | 07  m ==> 36 | 7 //返回小时，单双决定不足双数是否补零
 * ss == 36 | 07  s ==> 36 | 7 //返回秒数，单双决定不足双数是否补零
 * S ==> 478  返回毫秒数
 * EEE | EE | E //返回星期，对应格式 星期四 | 周四 | 四
 * e ==> 4 返回数值型星期
 * q ==> 4 返回第几季度
 * Q ==> 冬  返回春夏秋冬
 * X ==> 返回时间戳
 *
 */
class Moment {
  protected t: any;
  protected q: any;
  protected w: any;

  constructor() {
  }

  init(t: any) {
    this.t = t ? new Date(t) : new Date();
    this.q = ['\u6625', '\u590f', '\u79cb', '\u51ac'];
    this.w = ['\u65e5', '\u4e00', '\u4e8c', '\u4e09', '\u56db', '\u4e94', '\u516d'];
    return this;
  }

  format(fmt: any) {
    let o = {
      'M+': this.t.getMonth() + 1, // 月份
      'd+': this.t.getDate(), // 日
      'D+': this.t.getDate(), // 日
      'h+': this.t.getHours() % 12 === 0 ? 12 : this.t.getHours() % 12, // 小时
      'H+': this.t.getHours(), // 小时
      'm+': this.t.getMinutes(), // 分
      's+': this.t.getSeconds(), // 秒
      'q+': Math.floor((this.t.getMonth() + 3) / 3), // 季度
      'Q': this.q[Math.floor((this.t.getMonth() + 3) / 3)], // 返回春夏秋冬
      'S': this.t.getMilliseconds() // 毫秒
    };
    if (/(y+)/i.test(fmt)) {
      fmt = fmt.replace(RegExp.$1, (this.t.getFullYear() + '').substr(4 - RegExp.$1.length));
    }
    if (/(E+)/.test(fmt)) {
      fmt = fmt.replace(RegExp.$1,
        ((RegExp.$1.length > 1) ? (RegExp.$1.length > 2 ? '\u661f\u671f' : '\u5468') : '') + this.w[this.t.getDay()]);
    }
    if (/(e)/.test(fmt)) {
      fmt = fmt.replace(RegExp.$1, this.t.getDay());
    }
    if (/(X)/i.test(fmt)) {
      fmt = fmt.replace(RegExp.$1, this.t.getTime());
    }
    for (let k in o) {
      if (new RegExp('(' + k + ')').test(fmt)) {
        // @ts-ignore
        fmt = fmt.replace(RegExp.$1, (RegExp.$1.length === 1) ? (o[k]) : (('00' + o[k]).substr(('' + o[k]).length)));
      }
    }
    return fmt;
  }
}

function mo(t: any) {
  let moent = new Moment();
  return moent.init(t);
}
export default mo;
