import cr from './core';

const rbracket = /\[\]$/;

/**
 * 序列化对象或者数组
 * @param {string} prefix
 * @param obj
 * @param {boolean} traditional
 * @param {Function} add
 */
function buildParams(
  prefix: string, obj: any, traditional: boolean, add: Function) {
  let name;

  if (Array.isArray(obj)) {

    // Serialize array item.
    cr.each(obj, function(i: number | string, v: any) {
      if (traditional || rbracket.test(prefix)) {

        // Treat each array item as a scalar.
        add(prefix, v);

      } else {

        // Item is non-scalar (array or object), encode its numeric index.
        buildParams(
          prefix + '[' + (typeof v === 'object' && v != null ? i : '') + ']',
          v,
          traditional,
          add
        );
      }
      return true;
    });

  } else if (!traditional && cr.type(obj) === 'object') {

    // Serialize object item.
    for (name in obj) {
      if (obj.hasOwnProperty(name)) {
        buildParams(prefix + '[' + name + ']', obj[name], traditional, add);
      }
    }

  } else {

    // Serialize scalar item.
    add(prefix, obj);
  }
}

/**
 * urlEncoded格式序列化对象
 * @param data
 * @param {boolean} traditional
 * @returns {string}
 */
export function param(data: any, traditional: boolean = false) {
  let prefix,
    s: string[] = [],
    add = function(key: string, valueOrFunction: any) {

      // If value is a function, invoke it and use its return value
      let value = cr.isFunction(valueOrFunction) ? valueOrFunction() : valueOrFunction;

      s[s.length] = encodeURIComponent(key) + '=' +
        encodeURIComponent(value == null ? '' : value);
    };

  // If an array was passed in, assume that it is an array of form elements.
  if (Array.isArray(data) || (data.jquery && !cr.isPlainObject(data))) {

    // Serialize the form elements
    cr.each(data, function() {
      add(this.name, this.value);
      return true;
    });

  } else {

    // If traditional, encode the "old" way (the way 1.3.2 or older
    // did it), otherwise encode params recursively.
    for (prefix in data) {
      if (data.hasOwnProperty(prefix)) {
        buildParams(prefix, data[prefix], traditional, add);
      }
    }
  }

  // Return the resulting serialization
  return s.join('&');
}

/**
 * 对象创建FromData
 * @param data
 * @param formData
 * @param fob
 * @param prefix
 */
export function toFromData(
  data: any, formData?: FormData, fob: any = {}, prefix: string = ''): FormData {
  formData = formData || new FormData();
  let isObj: (data: any, fob: any, prefix: string) => void;
  isObj = function(data: any, fob: any = {}, prefix: string = '') {
    if (typeof data === 'object') {
      if (!!data && data.constructor === Array) {
        prefix += '[]';
        for (let i = 0; i < data.length; i++) {
          let auxFob = fob ? fob[i] : fob;
          isObj(data[i], auxFob, prefix);
        }
      } else if (data instanceof File || data instanceof Blob) {
        formData.append(prefix, data);
      } else {
        toFromData(data, formData, fob, prefix);
      }
    } else {
      let value = fob || data;
      formData.append(prefix, value);
    }
  };

  for (let prop in data) {
    if (data.hasOwnProperty(prop)) {
      let auxP = prefix === '' ? prop : (prefix + '[' + prop + ']');
      let auxFob = fob ? fob[prop] : fob;
      isObj(data[prop], auxFob, auxP);
    }
  }
  return formData;
}
