/**
 * created by liangchao on 2019/04/22.
 */
import cr from './core';

// 订阅者模式
/*
events: 订阅事件回调集合
{事件名称: {命名空间: [函数],实名函数名: [函数],匿名函数anonymous: [函数]}
icevents: 订阅事件回调集合
{事件名称: {命名空间: 函数,实名函数名: 函数,匿名函数anonymous: 函数}
on : 订阅事件
onin : 订阅后主动触发的事件，在某些情况下，订阅了一个事件后，需要执行相关的一些操作
emit: 发布消息
off: 取消订阅
 */

class Subscriber {
  private events: any;
  private icevents: any;
  private TE: any;

  constructor() {
    this.events = {};
    this.icevents = {};
    this.TE = {};
  }

  subscribe(type: any, listener: any, json?: any) {
    this.on(type, listener, json);
    let r = this.isReg(type);
    let a = r.a; // 事件名
    let b = r.b; // 命名空间
    let c = r.c; // 事件+命名空间
    if (a) {
      let te = this.TE[a[1]];
      if (te) {
        if (te.all) {
          listener.apply(this, te.all);
          return;
        }
        let c = null;
        cr.each(te, (d: any) => {
          c = d[0];
        });
        listener(c);
      }
      return;
    }

    if (b) {
      let te = this.TE[b[0]];
      if (te) {
        listener.apply(this, te);
      }
      return;
    }

    // 事件名加命名空间
    if (c) {
      let te = this.TE[c[1]];
      let tev = this.TE['.' + c[2]];
      if (te) {
        if (te[c[2]]) {
          listener.apply(this, te[c[2]]);
        } else if (te.all) {
          listener.apply(this, te.all);
        }
      } else if (tev) {
        listener.apply(this, tev);
      }
    }
  }

  on(type: any, listener: any, json?: any) {
    this.bind('events', [], type, listener, json);
  }

  onic(type: any, listener: any, json?: any) {
    this.bind('icevents', null, type, listener, json);
  }

  off(type: any, listener?: any) {
    // 如果没有传参数，关闭所有事件
    if (arguments.length === 0) {
      this.events = {};
      this.icevents = {};
      return;
    }
    let space = type.match(/^\.([^.]+)$/); // 命名空间
    let evSpace = type.match(/^([^.]+)\.([^.]+)$/); // 事件+命名空间
    let evName = type.match(/^([^.]+)$/); // 事件名
    let fnName = cr.getFnName(listener);

    // 事件名
    if (evName) {
      this.clearEvName(evName, fnName, listener, 'events', []);
      this.clearEvName(evName, fnName, listener, 'icevents', null);
    }
    // 命名空间的清除
    if (space) {
      this.clearSpace(space, 'events');
      this.clearSpace(space, 'icevents');
    }
    // 事件+命名空间情况下，清除该事件下的命名空间
    if (evSpace) {
      this.clearEvSpace(evSpace, 'events', []);
      this.clearEvSpace(evSpace, 'icevents', null);
    }
  }

  isReg(t: any) {
    return {
      a: t.match(/^([^.]+)$/),
      b: t.match(/^\.([^.]+)$/),
      c: t.match(/^([^.]+)\.([^.]+)$/)
    };
  }

  // 触发某个事件，不支持触发该事件中的某个事件，如命名空间，实名，匿名单独触发
  emit() {
    let args = Array.prototype.slice.apply(arguments);
    let AE = args.splice(1, args.length);
    let r = this.isReg(args[0]);
    let a = r.a; // 事件名
    let b = r.b; //  命名空间
    let c = r.c; // 事件+命名空间

    // 触发事件名
    if (a) {
      this.emitEventName(a, AE);
      return;
    }
    // 命名空间
    if (b) {
      this.emitSpaceName(b, AE);
      return;
    }
    // 事件+命名空间
    if (c) {
      this.emitEventSpace(c, AE);
    }
  }

  emitEventName(a: any, AE: any) {
    if (!this.TE[a[0]]) {
      this.TE[a[0]] = {};
    }
    this.TE[a[0]].all = AE;
    let es = this.events[a[0]];
    if (es) {
      cr.each(es, (d: any) => {
        cr.each(d, (e: any) => {
          e.apply(this, AE);
        });
      });
    }
    let cs = this.icevents[a[0]];
    if (cs) {
      cr.each(cs, (d: any) => {
        d.apply(this, AE);
      });
    }
  }

  emitSpaceName(b: any, AE: any) {
    this.TE[b[0]] = AE;
    cr.each(this.events, (d: any) => {
      cr.each(d, (e: any, v: any) => {
        // tslint:disable-next-line:triple-equals
        if (v == b[1]) {
          cr.each(e, (m: any) => {
            m.apply(this, AE);
          });
        }
      });
    });
    cr.each(this.icevents, (d: any) => {
      cr.each(d, (e: any, v: any) => {
        // tslint:disable-next-line:triple-equals
        if (v == b[1]) {
          e.apply(this, AE);
        }
      });
    });
  }

  emitEventSpace(c: any, AE: any) {
    if (!this.TE[c[1]]) {
      this.TE[c[1]] = {};
    }
    this.TE[c[1]][c[2]] = AE;
    let es = this.events[c[1]];
    if (es && es[c[2]]) {
      cr.each(es[c[2]], (d: any) => {
        d.apply(this, AE);
      });
    }
    let cs = this.icevents[c[1]];
    if (cs && cs[c[2]]) {
      cs[c[2]].apply(this, AE);
    }
  }

  // 主动触发订阅事件，在某些情况下，已经emit了，才on，trigger可以再次emit下，并且会把emit最后一次的参数保留
  trigger(type?: any) {
    let r = this.isReg(arguments[0]);
    let a = r.a; // 事件名
    let b = r.b; // 命名空间
    let c = r.c; // 事件+命名空间

    if (a) {
      this.triggerEventName(a);
      return;
    }

    if (b) {
      this.triggerSpaceName(b);
      return;
    }

    // 事件名加命名空间
    if (c) {
      this.triggerEventSpace(c);
    }
  }

  triggerEventName(a: any) {
    let te = this.TE[a[1]];
    let es = this.events[a[1]];
    let cs = this.icevents[a[1]];
    if (es) {
      cr.each(es, (d: any, v: any) => {
        if (te) {
          if (te[v]) {
            cr.each(d, (e: any) => {
              e.apply(this, te[v]);
            });
          } else if (te.all) {
            cr.each(d, (e: any) => {
              e.apply(this, te.all);
            });
          }
        } else {
          let tev = this.TE['.' + v];
          if (tev) {
            cr.each(d, (e: any) => {
              e.apply(this, tev);
            });
          }
        }

      });
    }
    if (cs) {
      cr.each(cs, (d: any, v: any) => {
        if (te) {
          if (te[v]) {
            d.apply(this, te[v]);
          } else if (te.all) {
            d.apply(this, te.all);
          }
        } else {
          let tev = this.TE['.' + v];
          if (tev) {
            d.apply(this, tev);
          }
        }
      });
    }
  }

  triggerSpaceName(b: any) {
    let te = this.TE[b[0]];
    cr.each(this.events, (d: any, v: any) => {
      if (!d[b[1]]) {
        return;
      }
      if (te) {
        cr.each(d[b[1]], (e: any) => {
          e.apply(this, te);
        });
      } else {
        let re = this.TE[v] ? this.TE[v][b[1]] || this.TE[v].all : '';
        if (re) {
          cr.each(d[b[1]], (e: any) => {
            e.apply(this, re);
          });
        }
      }
    });
    cr.each(this.icevents, (d: any, v: any) => {
      if (!d[b[1]]) {
        return;
      }
      if (te) {
        d[b[1]].apply(this, te);
      } else {
        let re = this.TE[v] ? this.TE[v].all : '';
        if (re) {
          d[b[1]].apply(this, re);
        }
      }
    });
  }

  triggerEventSpace(c: any) {
    let es = this.events[c[1]];
    let cs = this.icevents[c[1]];
    let te = this.TE[c[1]];
    let tev = this.TE['.' + c[2]];

    // 先判断事件是否存在
    if (es && es[c[2]]) {
      // 判断emit过的事件+命名空间
      if (te) {
        if (te[c[2]]) {
          cr.each(es[c[2]], (d: any) => {
            d.apply(this, te[c[2]]);
          });
        } else if (te.all) {
          cr.each(es[c[2]], (d: any) => {
            d.apply(this, te.all);
          });
        }
      } else if (tev) {
        cr.each(es[c[2]], (d: any) => {
          d.apply(this, tev);
        });
      }
    }

    if (cs && cs[c[2]]) {
      if (te) {
        if (te[c[2]]) {
          cs[c[2]].apply(this, te[c[2]]);
        } else if (te.all) {
          cs[c[2]].apply(this, te.all);
        }
      } else if (tev) {
        cs[c[2]].apply(this, tev);
      }
    }
  }

  bind(sv: any, r: any, type: any, listener: any, json?: any) {
    let settings = null,
      isFn = cr.type(listener),
      strs = type.match(/([^.]+)\.?([^.]+)?/);
    if (!strs) {
      return;
    }
    // 判断事件集合中是否存在
    // @ts-ignore
    if (!this[sv][strs[1]]) {
      // @ts-ignore
      this[sv][strs[1]] = {};
    }

    if (isFn === 'function') {
      // 判断是否是实名函数，实名函数优先于命名空间
      let fnName = cr.getFnName(listener);
      if (fnName) {
        this.isBind(sv, strs[1], fnName, r, listener);
      } else if (strs[2]) {
        // 非实名函数，有命名空间
        this.isBind(sv, strs[1], strs[2], r, listener);

      } else {
        // 匿名函数且无命名空间
        this.isBind(sv, strs[1], 'anonymous', r, listener);
      }
      settings = json;
    } else if (isFn === 'object') {
      settings = listener;
    }
    // 有订阅就会触发，方便在订阅某些事件需要执行的一系列操作
    // @ts-ignore
    if (cr.isFunction(this.onin)) {
      // @ts-ignore
      this.onin(strs[1], settings);
    }
  }

  isBind(s: any, s1: any, s2: any, r: any, f: any) {
    // @ts-ignore
    if (!this[s][s1][s2]) {
      // @ts-ignore
      this[s][s1][s2] = r;
    }
    if (cr.isArray(r)) {
      // @ts-ignore
      this[s][s1][s2].push(f);
    } else {
      // @ts-ignore
      this[s][s1][s2] = f;
    }
  }

  clearEvName(evName: any, fnName: any, listener: any, s: any, r: any) {
    // @ts-ignore
    if (!this[s][evName[1]]) {
      return;
    }
    if (fnName) {
      // @ts-ignore
      this[s][evName[1]][fnName] = r;
    } else {
      if (cr.isFunction(listener)) {
        // @ts-ignore
        if (cr.isArray(this[s][evName[1]].anonymous)) {
          cr.each(this.events[evName[1]].anonymous, (f: any, i: any) => {
            if (f === listener) {
              // @ts-ignore
              this[s][evName[1]].anonymous.splice(i, 1);
            }
          });
        } else {
          // @ts-ignore
          if (listener === this[s][evName[1]].anonymous) {
            // @ts-ignore
            this[s][evName[1]].anonymous = null;
          }
        }
      } else {
        // @ts-ignore
        this[s][evName[1]] = {};
      }
    }
  }

  clearSpace(space: any, s: any) {
    // @ts-ignore
    cr.each(this[s], (d: any) => {
      if (d[space[1]]) {
        d[space[1]] = [];
      }
    });
  }

  clearEvSpace(evSpace: any, s: any, r: any) {
    // @ts-ignore
    if (this[s][evSpace[1]] && this[s][evSpace[1]][evSpace[2]]) {
      // @ts-ignore
      this[s][evSpace[1]][evSpace[2]] = r;
    }
  }

}

export default Subscriber;
